searchData={"items":[{"type":"module","title":"Wafer","doc":"Welcome to Wafer. The funkiest way to write hardware drivers.\n\nThis library doesn't do much on it's own, it is used to help with some of the\nrepetitive tasks of writing drivers for hardware peripherals such as I2C and\nSPI connected sensors.\n\nWafer works with both ElixirALE and Circuits.  As such it's up to you to\ndefine which dependency you're using.","ref":"Wafer.html"},{"type":"protocol","title":"Wafer.Chip","doc":"A `Chip` is a physical peripheral with registers which can be read from and\nwritten to.\n\nRather than interacting with this protocol directly, it's a lot easier to use\nthe macros in `Wafer.Registers` to do it for you.","ref":"Wafer.Chip.html"},{"type":"protocol","title":"Deriving - Wafer.Chip","doc":"If you're implementing your own `Conn` type which simply delegates to one of\nthe lower level drivers then you can derive this protocol automatically:\n\n```elixir\ndefmodule MyConnection do\n  @derive Wafer.Chip\n  defstruct [:conn]\nend\n```\n\nIf your type uses a key other than `conn` for the inner connection you can\nspecify it while deriving:\n\n```elixir\ndefmodule MyConnection do\n  @derive {Wafer.Chip, key: :i2c_conn}\n  defstruct [:i2c_conn]\nend\n```","ref":"Wafer.Chip.html#module-deriving"},{"type":"protocol","title":"A note on SPI devices - Wafer.Chip","doc":"You will have to manually implement this protocol for SPI devices as there is\nno conventional way to read and write registers over SPI and every device has\ntheir own way of implementing an SPI instruction set.","ref":"Wafer.Chip.html#module-a-note-on-spi-devices"},{"type":"function","title":"Wafer.Chip.read_register/3","doc":"Read the register at the specified address.","ref":"Wafer.Chip.html#read_register/3"},{"type":"function","title":"Arguments - Wafer.Chip.read_register/3","doc":"- `conn` a type which implements the `Wafer.Conn` behaviour.\n  - `register_address` the address of the register to read from.\n  - `bytes` the number of bytes to read from the register.","ref":"Wafer.Chip.html#read_register/3-arguments"},{"type":"function","title":"Example - Wafer.Chip.read_register/3","doc":"iex> {:ok, conn} = ElixirALE.I2C.acquire(bus: \"i2c-1\", address: 0x68)\n    ...> Chip.read_register(conn, 0, 1)\n    {:ok,  >}","ref":"Wafer.Chip.html#read_register/3-example"},{"type":"function","title":"Wafer.Chip.swap_register/3","doc":"Perform a swap with the register at the specified address.  With some drivers\nthis is atomic, and with others it is implemented as a register read followed\nby a write.","ref":"Wafer.Chip.html#swap_register/3"},{"type":"function","title":"Arguments - Wafer.Chip.swap_register/3","doc":"- `conn` a type which implements the `Wafer.Conn` behaviour.\n  - `register_address` the address of the register to swap.\n  - `new_data` the data to write to the regsiter.","ref":"Wafer.Chip.html#swap_register/3-arguments"},{"type":"function","title":"Returns - Wafer.Chip.swap_register/3","doc":"The data that was previously in the register.","ref":"Wafer.Chip.html#swap_register/3-returns"},{"type":"function","title":"Example - Wafer.Chip.swap_register/3","doc":"iex> {:ok, conn} = ElixirALE.I2C.acquire(bus: \"i2c\", address: 0x68)\n    ...> Chip.swap_register(conn, 0,  >)\n    {:ok,  >, conn}","ref":"Wafer.Chip.html#swap_register/3-example"},{"type":"function","title":"Wafer.Chip.write_register/3","doc":"Write to the register at the specified address.","ref":"Wafer.Chip.html#write_register/3"},{"type":"function","title":"Arguments - Wafer.Chip.write_register/3","doc":"- `conn` a type which implements the `Wafer.Conn` behaviour.\n  - `register_address` the address of the register to write to.\n  - `data` a bitstring or binary of data to write to the register.","ref":"Wafer.Chip.html#write_register/3-arguments"},{"type":"function","title":"Example - Wafer.Chip.write_register/3","doc":"iex> {:ok, conn} = ElixirALE.I2C.acquire(bus: \"i2c\", address: 0x68)\n    ...> Chip.write_register(conn, 0,  >)\n    {:ok, conn}","ref":"Wafer.Chip.html#write_register/3-example"},{"type":"type","title":"Wafer.Chip.bytes/0","doc":"","ref":"Wafer.Chip.html#t:bytes/0"},{"type":"type","title":"Wafer.Chip.register_address/0","doc":"","ref":"Wafer.Chip.html#t:register_address/0"},{"type":"type","title":"Wafer.Chip.t/0","doc":"All the types that implement this protocol.","ref":"Wafer.Chip.html#t:t/0"},{"type":"behaviour","title":"Wafer.Conn","doc":"Defines a behaviour for connecting to a peripheral.\n\nThis behaviour is used by all the driver types in `Wafer` and you should\nimplement it for your devices also.","ref":"Wafer.Conn.html"},{"type":"behaviour","title":"Example - Wafer.Conn","doc":"Implementing `Conn` for a `HTS221` chip connected via Circuits' I2C driver.\n\n```elixir\ndefmodule HTS221 do\n  defstruct ~w[conn]a\n  alias Wafer.Driver.Circuits.I2C, as: Driver\n  @behaviour Wafer.Conn\n  @default_bus \"i2c-1\"\n  @default_address 0x5F\n\n  def acquire(opts) when is_list(opts) do\n    bus = Keyword.get(opts, :bus, @default_bus)\n    address = Keyword.get(opts, :address, @default_address)\n    with {:ok, conn} <- Driver.acquire(bus_name: bus, address: address),\n        do: {:ok, %HTS221{conn: conn}}\n  end\nend\n```","ref":"Wafer.Conn.html#module-example"},{"type":"callback","title":"Wafer.Conn.acquire/1","doc":"Acquire a connection to a peripheral using the provided driver.","ref":"Wafer.Conn.html#c:acquire/1"},{"type":"type","title":"Wafer.Conn.option/0","doc":"","ref":"Wafer.Conn.html#t:option/0"},{"type":"type","title":"Wafer.Conn.options/0","doc":"","ref":"Wafer.Conn.html#t:options/0"},{"type":"type","title":"Wafer.Conn.t/0","doc":"","ref":"Wafer.Conn.html#t:t/0"},{"type":"module","title":"Wafer.DLL","doc":"Implements a safe bytewise data-link-layer which allows the transmission and\nreceiption of arbitrary erlang terms in tiny packets.\n\nSee `Rx` and `Tx` for details.","ref":"Wafer.DLL.html"},{"type":"module","title":"Wafer.DLL.Rx","doc":"Bytewise reception buffer for our data link layer.","ref":"Wafer.DLL.Rx.html"},{"type":"function","title":"Wafer.DLL.Rx.complete?/1","doc":"Has the reception completed successfully?","ref":"Wafer.DLL.Rx.html#complete?/1"},{"type":"function","title":"Wafer.DLL.Rx.error?/1","doc":"Was there an error during reception?","ref":"Wafer.DLL.Rx.html#error?/1"},{"type":"function","title":"Wafer.DLL.Rx.init/0","doc":"Initialize a new empty buffer reading for receiving.","ref":"Wafer.DLL.Rx.html#init/0"},{"type":"function","title":"Wafer.DLL.Rx.rx/2","doc":"Receive a byte into the buffer.","ref":"Wafer.DLL.Rx.html#rx/2"},{"type":"function","title":"Wafer.DLL.Rx.value/1","doc":"Attempt to retrieve the received value from the buffer.","ref":"Wafer.DLL.Rx.html#value/1"},{"type":"type","title":"Wafer.DLL.Rx.t/0","doc":"","ref":"Wafer.DLL.Rx.html#t:t/0"},{"type":"module","title":"Wafer.DLL.Tx","doc":"A bytewise transmission buffer for our data link layer.","ref":"Wafer.DLL.Tx.html"},{"type":"function","title":"Wafer.DLL.Tx.complete?/1","doc":"","ref":"Wafer.DLL.Tx.html#complete?/1"},{"type":"function","title":"Wafer.DLL.Tx.init/1","doc":"Initialize a new transmission buffer from a term.","ref":"Wafer.DLL.Tx.html#init/1"},{"type":"function","title":"Wafer.DLL.Tx.tx/1","doc":"Get the next byte to transmit.","ref":"Wafer.DLL.Tx.html#tx/1"},{"type":"type","title":"Wafer.DLL.Tx.t/0","doc":"","ref":"Wafer.DLL.Tx.html#t:t/0"},{"type":"protocol","title":"Wafer.DeviceID","doc":"Allows devices to generate a unique identifier.\n\nMainly used for `InterruptRegistry` but you might find it useful too.  Can be\nderived in the same manner as all other `Wafer` protocols.","ref":"Wafer.DeviceID.html"},{"type":"function","title":"Wafer.DeviceID.id/1","doc":"Generate a unique identifier for `conn`.","ref":"Wafer.DeviceID.html#id/1"},{"type":"type","title":"Wafer.DeviceID.t/0","doc":"All the types that implement this protocol.","ref":"Wafer.DeviceID.html#t:t/0"},{"type":"module","title":"Wafer.Driver.Circuits.GPIO","doc":"A connection to a native GPIO pin via Circuits' GPIO driver.\n\nImplements the `Wafer.Conn` behaviour as well as the `Wafer.GPIO` protocol.","ref":"Wafer.Driver.Circuits.GPIO.html"},{"type":"function","title":"Wafer.Driver.Circuits.GPIO.acquire/1","doc":"Acquire a connection to a native GPIO pin via Circuit's GPIO driver.","ref":"Wafer.Driver.Circuits.GPIO.html#acquire/1"},{"type":"function","title":"Options - Wafer.Driver.Circuits.GPIO.acquire/1","doc":"- `:pin` (required) the integer number of the pin to connect to.  Hardware dependent.\n  - `:direction` (optional) either `:in` or `:out`.  Defaults to `:out`.","ref":"Wafer.Driver.Circuits.GPIO.html#acquire/1-options"},{"type":"type","title":"Wafer.Driver.Circuits.GPIO.option/0","doc":"","ref":"Wafer.Driver.Circuits.GPIO.html#t:option/0"},{"type":"type","title":"Wafer.Driver.Circuits.GPIO.options/0","doc":"","ref":"Wafer.Driver.Circuits.GPIO.html#t:options/0"},{"type":"type","title":"Wafer.Driver.Circuits.GPIO.t/0","doc":"","ref":"Wafer.Driver.Circuits.GPIO.html#t:t/0"},{"type":"module","title":"Wafer.Driver.Circuits.GPIO.Dispatcher","doc":"This module implements a simple dispatcher for GPIO interrupts when using\n`Circuits.GPIO`.\n\nBecause the Circuit's interrupt doesn't provide an indication of whether the\npin is rising or falling we store the last known pin state and use it to\ncompare.","ref":"Wafer.Driver.Circuits.GPIO.Dispatcher.html"},{"type":"function","title":"Wafer.Driver.Circuits.GPIO.Dispatcher.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Wafer.Driver.Circuits.GPIO.Dispatcher.html#child_spec/1"},{"type":"function","title":"Wafer.Driver.Circuits.GPIO.Dispatcher.disable/2","doc":"Disable interrupts for this connection using the specified pin_condition.","ref":"Wafer.Driver.Circuits.GPIO.Dispatcher.html#disable/2"},{"type":"function","title":"Wafer.Driver.Circuits.GPIO.Dispatcher.enable/3","doc":"Enable interrupts for this connection using the specified pin_condition.","ref":"Wafer.Driver.Circuits.GPIO.Dispatcher.html#enable/3"},{"type":"module","title":"Wafer.Driver.Circuits.I2C","doc":"A connection to a chip via Circuits' I2C driver.\n\nImplements the `Wafer.Conn` behaviour as well as the `Wafer.Chip` and `Wafer.I2C` protocols.","ref":"Wafer.Driver.Circuits.I2C.html"},{"type":"function","title":"Wafer.Driver.Circuits.I2C.acquire/1","doc":"Acquire a connection to a peripheral using the Circuits' I2C driver on the\nspecified bus and address.","ref":"Wafer.Driver.Circuits.I2C.html#acquire/1"},{"type":"type","title":"Wafer.Driver.Circuits.I2C.option/0","doc":"","ref":"Wafer.Driver.Circuits.I2C.html#t:option/0"},{"type":"type","title":"Wafer.Driver.Circuits.I2C.options/0","doc":"","ref":"Wafer.Driver.Circuits.I2C.html#t:options/0"},{"type":"type","title":"Wafer.Driver.Circuits.I2C.t/0","doc":"","ref":"Wafer.Driver.Circuits.I2C.html#t:t/0"},{"type":"module","title":"Wafer.Driver.Circuits.SPI","doc":"A connection to a chip via Circuits's SPI driver.\n\nImplements the `Wafer.Conn` behaviour as well as the `Wafer.SPI` protocol.","ref":"Wafer.Driver.Circuits.SPI.html"},{"type":"function","title":"Wafer.Driver.Circuits.SPI.acquire/1","doc":"Acquire a connection to a peripheral using the Circuits' SPI driver on the\nspecified bus and address.","ref":"Wafer.Driver.Circuits.SPI.html#acquire/1"},{"type":"type","title":"Wafer.Driver.Circuits.SPI.driver_option/0","doc":"","ref":"Wafer.Driver.Circuits.SPI.html#t:driver_option/0"},{"type":"type","title":"Wafer.Driver.Circuits.SPI.option/0","doc":"","ref":"Wafer.Driver.Circuits.SPI.html#t:option/0"},{"type":"type","title":"Wafer.Driver.Circuits.SPI.options/0","doc":"","ref":"Wafer.Driver.Circuits.SPI.html#t:options/0"},{"type":"type","title":"Wafer.Driver.Circuits.SPI.t/0","doc":"","ref":"Wafer.Driver.Circuits.SPI.html#t:t/0"},{"type":"module","title":"Wafer.Driver.ElixirALE.GPIO","doc":"A connection to a native GPIO pin via ElixirALE's GPIO driver.\n\nImplements the `Wafer.Conn` behaviour as well as the `Wafer.GPIO` protocol.","ref":"Wafer.Driver.ElixirALE.GPIO.html"},{"type":"function","title":"Wafer.Driver.ElixirALE.GPIO.acquire/1","doc":"Acquire a connection to the specified GPIO pin using the ElixirALE GPIO driver.","ref":"Wafer.Driver.ElixirALE.GPIO.html#acquire/1"},{"type":"function","title":"Options - Wafer.Driver.ElixirALE.GPIO.acquire/1","doc":"- `:pin` (required) - the integer pin number.  Hardware dependent.\n  - `:direction` - either `:in` or `:out`.  Defaults to `:out`.","ref":"Wafer.Driver.ElixirALE.GPIO.html#acquire/1-options"},{"type":"type","title":"Wafer.Driver.ElixirALE.GPIO.option/0","doc":"","ref":"Wafer.Driver.ElixirALE.GPIO.html#t:option/0"},{"type":"type","title":"Wafer.Driver.ElixirALE.GPIO.options/0","doc":"","ref":"Wafer.Driver.ElixirALE.GPIO.html#t:options/0"},{"type":"type","title":"Wafer.Driver.ElixirALE.GPIO.t/0","doc":"","ref":"Wafer.Driver.ElixirALE.GPIO.html#t:t/0"},{"type":"module","title":"Wafer.Driver.ElixirALE.GPIO.Dispatcher","doc":"This module implements a simple dispatcher for GPIO interrupts when using\n`ElixirALE`.","ref":"Wafer.Driver.ElixirALE.GPIO.Dispatcher.html"},{"type":"function","title":"Wafer.Driver.ElixirALE.GPIO.Dispatcher.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Wafer.Driver.ElixirALE.GPIO.Dispatcher.html#child_spec/1"},{"type":"function","title":"Wafer.Driver.ElixirALE.GPIO.Dispatcher.disable/2","doc":"Disable interrupts for this connection on the specified pin_condition.","ref":"Wafer.Driver.ElixirALE.GPIO.Dispatcher.html#disable/2"},{"type":"function","title":"Wafer.Driver.ElixirALE.GPIO.Dispatcher.enable/3","doc":"Enable interrupts for this connection using the specified pin_condition.","ref":"Wafer.Driver.ElixirALE.GPIO.Dispatcher.html#enable/3"},{"type":"module","title":"Wafer.Driver.ElixirALE.I2C","doc":"A connection to a chip via ElixirALE's I2C driver.\n\nImplements the `Wafer.Conn` behaviour as well as the `Wafer.Chip` and `Wafer.I2C` protocols.","ref":"Wafer.Driver.ElixirALE.I2C.html"},{"type":"function","title":"Wafer.Driver.ElixirALE.I2C.acquire/1","doc":"Acquire a connection to a peripheral using the ElixirALE I2C driver on the\nspecified bus and address.","ref":"Wafer.Driver.ElixirALE.I2C.html#acquire/1"},{"type":"type","title":"Wafer.Driver.ElixirALE.I2C.option/0","doc":"","ref":"Wafer.Driver.ElixirALE.I2C.html#t:option/0"},{"type":"type","title":"Wafer.Driver.ElixirALE.I2C.options/0","doc":"","ref":"Wafer.Driver.ElixirALE.I2C.html#t:options/0"},{"type":"type","title":"Wafer.Driver.ElixirALE.I2C.t/0","doc":"","ref":"Wafer.Driver.ElixirALE.I2C.html#t:t/0"},{"type":"module","title":"Wafer.Driver.ElixirALE.SPI","doc":"A connection to a chip via ElixirALE's SPI driver.\n\nImplements the `Wafer.Conn` behaviour as well as the `Wafer.SPI` protocol.","ref":"Wafer.Driver.ElixirALE.SPI.html"},{"type":"function","title":"Wafer.Driver.ElixirALE.SPI.acquire/1","doc":"Acquire a connection to a peripheral using the ElixirALE' SPI driver on the\nspecified bus and address.","ref":"Wafer.Driver.ElixirALE.SPI.html#acquire/1"},{"type":"type","title":"Wafer.Driver.ElixirALE.SPI.driver_option/0","doc":"","ref":"Wafer.Driver.ElixirALE.SPI.html#t:driver_option/0"},{"type":"type","title":"Wafer.Driver.ElixirALE.SPI.option/0","doc":"","ref":"Wafer.Driver.ElixirALE.SPI.html#t:option/0"},{"type":"type","title":"Wafer.Driver.ElixirALE.SPI.options/0","doc":"","ref":"Wafer.Driver.ElixirALE.SPI.html#t:options/0"},{"type":"type","title":"Wafer.Driver.ElixirALE.SPI.t/0","doc":"","ref":"Wafer.Driver.ElixirALE.SPI.html#t:t/0"},{"type":"module","title":"Wafer.Driver.Fake","doc":"This module provides a fake driver which you can use in your tests.\n\nIt doesn't really do anything except return zeroes or empty results.  I\nsuggest that you [use Mimic](https://hex.pm/packages/mimic) to set\nexpectations for function calls and mock return values.\n\nThis module implements the `Chip`, `DeviceID`, `GPIO`, `I2C` and `SPI`\nprotocols.","ref":"Wafer.Driver.Fake.html"},{"type":"type","title":"Wafer.Driver.Fake.t/0","doc":"","ref":"Wafer.Driver.Fake.html#t:t/0"},{"type":"module","title":"Wafer.Format","doc":"Handy functions for formatting bytes, especially for debugging.","ref":"Wafer.Format.html"},{"type":"function","title":"Wafer.Format.to_bin/1","doc":"Convert the provided value into it's 0-padded byte-oriented string\nrepresentation.","ref":"Wafer.Format.html#to_bin/1"},{"type":"function","title":"Examples - Wafer.Format.to_bin/1","doc":"iex> to_bin(0x1234)\n    \"0b00010010_00110100\"\n\n    iex> to_bin( >)\n    \"0b11110000_00001111\"","ref":"Wafer.Format.html#to_bin/1-examples"},{"type":"function","title":"Wafer.Format.to_hex/1","doc":"Convert the provided value into it's 0-padded byte-oriented string\nrepresentation.","ref":"Wafer.Format.html#to_hex/1"},{"type":"function","title":"Examples - Wafer.Format.to_hex/1","doc":"iex> to_hex(0x1234)\n    \"0x1234\"\n\n    iex> to_hex( >)\n    \"0xF00F\"","ref":"Wafer.Format.html#to_hex/1-examples"},{"type":"protocol","title":"Wafer.GPIO","doc":"A `GPIO` is a physical pin which can be read from and written to.\n\nSome hardware supports interrupts, some has internal pull up/down resistors.\nWafer supports all of these, however not all drivers do.","ref":"Wafer.GPIO.html"},{"type":"protocol","title":"Deriving - Wafer.GPIO","doc":"If you're implementing your own `Conn` type that simply delegates to one of\nthe lower level drivers then you can derive this protocol automatically:\n\n```elixir\ndefstruct MyPin do\n  @derive Wafer.GPIO\n  defstruct [:conn]\nend\n```\n\nIf your type uses a key other than `conn` for the inner connection you can\nspecify it while deriving:\n\n```elixir\ndefstruct MyPin do\n  @derive {Wafer.GPIO, key: :gpio_conn}\n  defstruct [:gpio_conn]\nend\n```","ref":"Wafer.GPIO.html#module-deriving"},{"type":"function","title":"Wafer.GPIO.direction/2","doc":"Set the pin direction.","ref":"Wafer.GPIO.html#direction/2"},{"type":"function","title":"Wafer.GPIO.disable_interrupt/2","doc":"Disables interrupts for this connection and pin_condition.","ref":"Wafer.GPIO.html#disable_interrupt/2"},{"type":"function","title":"Wafer.GPIO.enable_interrupt/3","doc":"Enable an interrupt for this connection and pin_condition.","ref":"Wafer.GPIO.html#enable_interrupt/3"},{"type":"function","title":"Arguments - Wafer.GPIO.enable_interrupt/3","doc":"- `pin_condition` - Either `:rising` or `:falling`.\n- `metadata` - any optional data you wish to receive along with your\n  interrupts.  Defaults to `nil`.\n\nInterrupts will be sent to the calling process as messages in the form of\n`{:interrupt, Conn.t(), pin_condition, metadata | nil}`.","ref":"Wafer.GPIO.html#enable_interrupt/3-arguments"},{"type":"function","title":"Implementors note - Wafer.GPIO.enable_interrupt/3","doc":"`Wafer` starts it's own `Registry` named `Wafer.InterruptRegistry` which you\ncan use to publish your interrupts to using the above format.  The registry\nkey is set as follows: `{PublishingModule, pin, pin_condition}`.  You can see\nexamples in the `Circuits.GPIO.Dispatcher` and `ElixirALE.GPIO.Dispatcher`\nmodules.","ref":"Wafer.GPIO.html#enable_interrupt/3-implementors-note"},{"type":"function","title":"Wafer.GPIO.pull_mode/2","doc":"Set the pull mode for this pin.\n\nIf the hardware contains software-switchable pull-up and/or pull-down\nresistors you can configure them this way.  If they are not supported then\nthis function will return `{:error, :not_supported}`.","ref":"Wafer.GPIO.html#pull_mode/2"},{"type":"function","title":"Wafer.GPIO.read/1","doc":"Read the current pin value.","ref":"Wafer.GPIO.html#read/1"},{"type":"function","title":"Wafer.GPIO.write/2","doc":"Set the pin value.","ref":"Wafer.GPIO.html#write/2"},{"type":"type","title":"Wafer.GPIO.interrupt_option/0","doc":"","ref":"Wafer.GPIO.html#t:interrupt_option/0"},{"type":"type","title":"Wafer.GPIO.interrupt_options/0","doc":"","ref":"Wafer.GPIO.html#t:interrupt_options/0"},{"type":"type","title":"Wafer.GPIO.pin_condition/0","doc":"","ref":"Wafer.GPIO.html#t:pin_condition/0"},{"type":"type","title":"Wafer.GPIO.pin_direction/0","doc":"","ref":"Wafer.GPIO.html#t:pin_direction/0"},{"type":"type","title":"Wafer.GPIO.pin_number/0","doc":"","ref":"Wafer.GPIO.html#t:pin_number/0"},{"type":"type","title":"Wafer.GPIO.pin_value/0","doc":"","ref":"Wafer.GPIO.html#t:pin_value/0"},{"type":"type","title":"Wafer.GPIO.pull_mode/0","doc":"","ref":"Wafer.GPIO.html#t:pull_mode/0"},{"type":"type","title":"Wafer.GPIO.t/0","doc":"All the types that implement this protocol.","ref":"Wafer.GPIO.html#t:t/0"},{"type":"module","title":"Wafer.Guards","doc":"Handy guards which you can use in your code to assert correct values.","ref":"Wafer.Guards.html"},{"type":"macro","title":"Wafer.Guards.is_byte_size/1","doc":"Byte sizes are non negative integers","ref":"Wafer.Guards.html#is_byte_size/1"},{"type":"macro","title":"Wafer.Guards.is_i2c_address/1","doc":"An integer between `0` and `0x7F`","ref":"Wafer.Guards.html#is_i2c_address/1"},{"type":"macro","title":"Wafer.Guards.is_pin_condition/1","doc":"One of `:none`, `:rising`, `:falling` or `:both`","ref":"Wafer.Guards.html#is_pin_condition/1"},{"type":"macro","title":"Wafer.Guards.is_pin_direction/1","doc":"Either `:in` or `:out`","ref":"Wafer.Guards.html#is_pin_direction/1"},{"type":"macro","title":"Wafer.Guards.is_pin_number/1","doc":"Pin numbers are non negative integers","ref":"Wafer.Guards.html#is_pin_number/1"},{"type":"macro","title":"Wafer.Guards.is_pin_pull_mode/1","doc":"One of `:not_set`, `:none`, `:pull_up` or `:pull_down`","ref":"Wafer.Guards.html#is_pin_pull_mode/1"},{"type":"macro","title":"Wafer.Guards.is_pin_value/1","doc":"Either `0` or `1`","ref":"Wafer.Guards.html#is_pin_value/1"},{"type":"macro","title":"Wafer.Guards.is_register_address/1","doc":"Register addresses are non negative integers usually only one byte, but we don't enforce that here","ref":"Wafer.Guards.html#is_register_address/1"},{"type":"protocol","title":"Wafer.I2C","doc":"A protocol for interacting with I2C devices directly.  Most of the time you'll\nwant to use the `Chip` protocol for working with registers, but this is\nprovided for consistency's sake.\n\nThis API is extremely similar to the `ElixirALE.I2C` and `Circuits.I2C` APIs,\nexcept that it takes a `Conn` which implements `I2C` as an argument.","ref":"Wafer.I2C.html"},{"type":"protocol","title":"Deriving - Wafer.I2C","doc":"If you're implementing your own `Conn` type that simply delegates to one of\nthe lower level drivers then you can derive this protocol automatically:\n\n```elixir\ndefstruct MyI2CDevice do\n  @derive Wafer.I2C\n  defstruct [:conn]\nend\n```\n\nIf your type uses a key other than `conn` for the inner connection you can\nspecify it while deriving:\n\n```elixir\ndefstruct MyI2CDevice do\n  @derive {Wafer.I2C, key: :i2c_conn}\n  defstruct [:i2c_conn]\nend","ref":"Wafer.I2C.html#module-deriving"},{"type":"function","title":"Wafer.I2C.detect_devices/1","doc":"Detect the devices adjacent to the connection's device on the same I2C bus.","ref":"Wafer.I2C.html#detect_devices/1"},{"type":"function","title":"Wafer.I2C.read/3","doc":"Initiate a read transaction to the connection's I2C device.","ref":"Wafer.I2C.html#read/3"},{"type":"function","title":"Wafer.I2C.write/3","doc":"Write `data` to the connection's I2C device.","ref":"Wafer.I2C.html#write/3"},{"type":"function","title":"Wafer.I2C.write_read/4","doc":"Write data to an I2C device and then immediately issue a read.","ref":"Wafer.I2C.html#write_read/4"},{"type":"type","title":"Wafer.I2C.address/0","doc":"","ref":"Wafer.I2C.html#t:address/0"},{"type":"type","title":"Wafer.I2C.data/0","doc":"","ref":"Wafer.I2C.html#t:data/0"},{"type":"type","title":"Wafer.I2C.option/0","doc":"","ref":"Wafer.I2C.html#t:option/0"},{"type":"type","title":"Wafer.I2C.options/0","doc":"","ref":"Wafer.I2C.html#t:options/0"},{"type":"type","title":"Wafer.I2C.t/0","doc":"All the types that implement this protocol.","ref":"Wafer.I2C.html#t:t/0"},{"type":"module","title":"Wafer.InterruptRegistry","doc":"This module provides Wafer's interrupt registry.  This allows multiple\nsubscribers to be subscribed to interrupts from many different pins.\n\nIt is used by `Driver.Circuits.GPIO.Dispatcher` and\n`Driver.ElixirALE.GPIO.Dispatcher` and you should probably use it if you're\nwriting your own driver which supports sending interrupts to subscribers.","ref":"Wafer.InterruptRegistry.html"},{"type":"function","title":"Wafer.InterruptRegistry.count_subscriptions/1","doc":"Count the number of active subscriptions for `key`.","ref":"Wafer.InterruptRegistry.html#count_subscriptions/1"},{"type":"function","title":"Wafer.InterruptRegistry.count_subscriptions/2","doc":"Count the number of active subscriptions for `key` and `pin_condition`.","ref":"Wafer.InterruptRegistry.html#count_subscriptions/2"},{"type":"function","title":"Wafer.InterruptRegistry.publish/2","doc":"Publish an interrupt to subscribers that are interested in `key` and\n`pin_condition`.\n\nSearches the registry for all subscribers to are subscribed to `pin_condition`\nor `:both` and publishes the interrupt message to them.  The interrupt message\ntakes the form `{:interrupt, Conn.t(), :rising | :falling, metadata :: any}`.","ref":"Wafer.InterruptRegistry.html#publish/2"},{"type":"function","title":"Wafer.InterruptRegistry.subscribe/3","doc":"Subscribe the calling process to interrupts for the specified `pin_condition`\nusing the provided `key`.\n\nSee `subscribe/4` for more information.","ref":"Wafer.InterruptRegistry.html#subscribe/3"},{"type":"function","title":"Example - Wafer.InterruptRegistry.subscribe/3","doc":"iex> subscribe({MCP230017.Pin, 13}, :rising, conn)\n    :ok","ref":"Wafer.InterruptRegistry.html#subscribe/3-example"},{"type":"function","title":"Wafer.InterruptRegistry.subscribe/4","doc":"Subscribe the calling process to interrupts for the specified `pin_condition`\nusing the provided `key`.","ref":"Wafer.InterruptRegistry.html#subscribe/4"},{"type":"function","title":"Arguments - Wafer.InterruptRegistry.subscribe/4","doc":"- `key` a key which uniquely describes the pin.  Probably a combination of\n    device name and pin.\n  - `pin_condition` either `:rising`, `:falling` or `:both`.\n  - `conn` the receiver's connection to the pin, sent back to them in the\n    interrupt message.\n  - `metadata` arbitrary data which will be sent to the receiver process in\n    the interrupt message. Defaults to `nil`.","ref":"Wafer.InterruptRegistry.html#subscribe/4-arguments"},{"type":"function","title":"Wafer.InterruptRegistry.subscribers?/1","doc":"Are there any subscribers to `key`?","ref":"Wafer.InterruptRegistry.html#subscribers?/1"},{"type":"function","title":"Wafer.InterruptRegistry.subscribers?/2","doc":"Are there any subscribers to `key` and `pin_condition`?","ref":"Wafer.InterruptRegistry.html#subscribers?/2"},{"type":"function","title":"Wafer.InterruptRegistry.subscriptions/1","doc":"Returns a list of all subscriptions to `key`.","ref":"Wafer.InterruptRegistry.html#subscriptions/1"},{"type":"function","title":"Wafer.InterruptRegistry.subscriptions/2","doc":"Returns a list of all subscriptions `key` and `pin_condition`.","ref":"Wafer.InterruptRegistry.html#subscriptions/2"},{"type":"function","title":"Wafer.InterruptRegistry.unsubscribe/3","doc":"Remove all subscriptions for `key`, `pin_condition` and `conn`.","ref":"Wafer.InterruptRegistry.html#unsubscribe/3"},{"type":"type","title":"Wafer.InterruptRegistry.key/0","doc":"","ref":"Wafer.InterruptRegistry.html#t:key/0"},{"type":"module","title":"Wafer.Registers","doc":"This module provides helpful macros for specifying the registers used to\ncommunicate with your device.\n\nThis can be a massive time saver, and means you can basically just copy them\nstraight out of the datasheet.\n\nSee the documentation for `defregister/4` for more information.","ref":"Wafer.Registers.html"},{"type":"macro","title":"Wafer.Registers.defregister/3","doc":"Define functions for interacting with a device register with common defaults.","ref":"Wafer.Registers.html#defregister/3"},{"type":"macro","title":"Examples - Wafer.Registers.defregister/3","doc":"When specified with an access mode, assumes a 1 byte register:\n\n    iex> defregister(:status, 0x03, :ro)\n\nWhen specified with a byte size, assumes a `:rw` register:\n\n    iex> defregister(:config, 0x02, 2)","ref":"Wafer.Registers.html#defregister/3-examples"},{"type":"macro","title":"Wafer.Registers.defregister/4","doc":"Define functions for interacting with a device register.","ref":"Wafer.Registers.html#defregister/4"},{"type":"macro","title":"Parameters - Wafer.Registers.defregister/4","doc":"- `name` - name of the register.\n  - `register_address` - the address of the register.\n  - `mode` the access mode of the register.\n  - `bytes` the number of bytes in the register.","ref":"Wafer.Registers.html#defregister/4-parameters"},{"type":"macro","title":"Examples - Wafer.Registers.defregister/4","doc":"#","ref":"Wafer.Registers.html#defregister/4-examples"},{"type":"macro","title":"Read-only registers - Wafer.Registers.defregister/4","doc":"Define a read-only register named `status` at address `0x03` which is a single\nbyte wide:\n\n    iex> defregister(:status, 0x03, :ro, 1)\n\nThis will define the following function along with documentation and\ntypespecs:\n\n```elixir\ndef read_status(conn), do: Chip.read_register(conn, 0x03, 1)\n```\n\n#","ref":"Wafer.Registers.html#defregister/4-read-only-registers"},{"type":"macro","title":"Write-only registers - Wafer.Registers.defregister/4","doc":"Define a write-only register named `int_en` at address `0x02` which is 2 bytes\nwide:\n\n    iex> defregister(:int_en, 0x02, :wo, 2)\n\nThis will define the following functions along with documentation and\ntypespecs:\n\n```elixir\ndef write_int_en(conn, data)\n    when is_binary(data) and byte_size(data) == 2,\n    do: Chip.write_register(conn, 0x2, data)\n\ndef write_int_en(_conn, data),\n    do: {:error, \"Argument error: #{inspect(data)}\"}\n```\n\n#","ref":"Wafer.Registers.html#defregister/4-write-only-registers"},{"type":"macro","title":"Read-write registers. - Wafer.Registers.defregister/4","doc":"Define a read-write register named `config` at address `0x01`.\n\n    iex> defregister(:config, 0x01, :rw, 1)\n\nIn addition to defining `read_config/1` and `write_config/2` as per the\nexamples above it will also generate the following functions along with\ndocumentation and typespecs:\n\n```elixir\ndef swap_config(conn, data)\n    when is_binary(data) and byte_size(data) == 1,\n    do: Chip.swap_register(conn, 0x01, data)\n\ndef swap_config(_conn, data),\n    do: {:error, \"Argument error: #{inspect(data)}\"}\n\ndef update_config(conn, callback)\n    when is_function(callback, 1) do\n      with {:ok, data} <- Chip.read_register(conn, 0x01, 1),\n           new_data when is_binary(new_data) and byte_size(new_data) == 1 <- callback.(data),\n           {:ok, conn} <- Chip.write_regsiter(conn, 0x01, new_data),\n           do: {:ok, conn}\n    end\n\ndef update_config(_conn, _callback),\n    do: {:error, \"Argument error: callback should be an arity 1 function\"}\n```","ref":"Wafer.Registers.html#defregister/4-read-write-registers"},{"type":"type","title":"Wafer.Registers.access_mode/0","doc":"","ref":"Wafer.Registers.html#t:access_mode/0"},{"type":"type","title":"Wafer.Registers.bytes/0","doc":"","ref":"Wafer.Registers.html#t:bytes/0"},{"type":"type","title":"Wafer.Registers.register_name/0","doc":"","ref":"Wafer.Registers.html#t:register_name/0"},{"type":"protocol","title":"Wafer.Release","doc":"A protocol for releasing connections.  The opposite of `Conn`'s `acquire/1`.","ref":"Wafer.Release.html"},{"type":"protocol","title":"Deriving - Wafer.Release","doc":"If you're implementing your own `Conn` type that simply delegates to one of\nthe lower level drivers that you can derive this protocol automatically:\n\n```elixir\ndefstruct MyConn do\n  @derive Wafer.Release\n  defstruct [:conn]\nend\n```\n\nIf your type uses a key other than `conn` for the inner connection you can specify it while deriving:\n\n```elixir\ndefstruct MyConn do\n  @derive {Wafer.Release, key: :pin_conn}\n  defstruct [:pin_conn]\nend\n```","ref":"Wafer.Release.html#module-deriving"},{"type":"function","title":"Wafer.Release.release/1","doc":"Release all resources associated with the connection.  Usually in preparation\nfor shutdown.","ref":"Wafer.Release.html#release/1"},{"type":"type","title":"Wafer.Release.t/0","doc":"All the types that implement this protocol.","ref":"Wafer.Release.html#t:t/0"},{"type":"protocol","title":"Wafer.SPI","doc":"A (very simple) protocol for interacting with SPI connected devices.\n\nThis API is a minimal version of the `ElixirALE.SPI` and `Circuits.SPI` APIs,\nexcept that it takes a `Conn` which implements `SPI` as an argument.  If you\nwant to use any advanced features, such as bus detection, I advise you to\ninteract with the underlying driver directly.","ref":"Wafer.SPI.html"},{"type":"protocol","title":"Deriving - Wafer.SPI","doc":"If you're implementing your own `Conn` type that simply delegates to one of\nthe lower level drivers then you can derive this protocol automatically:\n\n```elixir\ndefstruct MySPIDevice do\n  @derive Wafer.SPI\n  defstruct [:conn]\nend\n```\n\nIf your type uses a key other than `conn` for the inner connection you can\nspecify it while deriving:\n\n```elixir\ndefstruct MySPIDevice do\n  @derive {Wafer.SPI, key: :spi_conn}\n  defstruct [:spi_conn]\nend","ref":"Wafer.SPI.html#module-deriving"},{"type":"function","title":"Wafer.SPI.transfer/2","doc":"Perform an SPI transfer.\n\nSPI transfers are synchronous, so `data` should be a binary of bytes to send\nto the device, and you will receive back a binary of the same length\ncontaining the data received from the device.","ref":"Wafer.SPI.html#transfer/2"},{"type":"type","title":"Wafer.SPI.data/0","doc":"","ref":"Wafer.SPI.html#t:data/0"},{"type":"type","title":"Wafer.SPI.t/0","doc":"All the types that implement this protocol.","ref":"Wafer.SPI.html#t:t/0"},{"type":"module","title":"Wafer.Twiddles","doc":"Handy functions for dealing with bits and bytes in the wild.","ref":"Wafer.Twiddles.html"},{"type":"function","title":"Wafer.Twiddles.clear_bit/2","doc":"Set the specified `bit` to `0` within the `byte`.","ref":"Wafer.Twiddles.html#clear_bit/2"},{"type":"function","title":"Example - Wafer.Twiddles.clear_bit/2","doc":"iex> clear_bit(3, 0)\n    2\n\n    iex> clear_bit( >, 0)\n     >","ref":"Wafer.Twiddles.html#clear_bit/2-example"},{"type":"function","title":"Wafer.Twiddles.count_ones/1","doc":"Returns the number of 1's in `byte`.","ref":"Wafer.Twiddles.html#count_ones/1"},{"type":"function","title":"Example - Wafer.Twiddles.count_ones/1","doc":"iex> count_ones(0x7f)\n    7\n\n    iex> count_ones( >)\n    7","ref":"Wafer.Twiddles.html#count_ones/1-example"},{"type":"function","title":"Wafer.Twiddles.count_zeroes/1","doc":"Returns the number of 0's in `byte`.","ref":"Wafer.Twiddles.html#count_zeroes/1"},{"type":"function","title":"Example - Wafer.Twiddles.count_zeroes/1","doc":"iex> count_zeroes(0x7f)\n    1\n\n    iex> count_zeroes( >)\n    1","ref":"Wafer.Twiddles.html#count_zeroes/1-example"},{"type":"function","title":"Wafer.Twiddles.find_ones/1","doc":"Find all the `1` bits in `byte` and return a list of `bit_number`s.","ref":"Wafer.Twiddles.html#find_ones/1"},{"type":"function","title":"Example - Wafer.Twiddles.find_ones/1","doc":"iex> find_ones(0x0A)\n    [1, 3]\n\n    iex> find_ones( >)\n    [1, 3]","ref":"Wafer.Twiddles.html#find_ones/1-example"},{"type":"function","title":"Wafer.Twiddles.find_zeroes/1","doc":"Find all the `0` bits in `byte` and return a list of `bit_number`s.","ref":"Wafer.Twiddles.html#find_zeroes/1"},{"type":"function","title":"Example - Wafer.Twiddles.find_zeroes/1","doc":"iex> find_zeroes(0xFA)\n    [0, 2]\n\n    iex> find_zeroes( >)\n    [0, 2]","ref":"Wafer.Twiddles.html#find_zeroes/1-example"},{"type":"function","title":"Wafer.Twiddles.get_bit/2","doc":"Get the bit at the specified `bit_number` within `byte`.","ref":"Wafer.Twiddles.html#get_bit/2"},{"type":"function","title":"Example - Wafer.Twiddles.get_bit/2","doc":"iex> get_bit(0x7f, 6)\n    1\n\n    iex> get_bit( >, 6)\n    1","ref":"Wafer.Twiddles.html#get_bit/2-example"},{"type":"function","title":"Wafer.Twiddles.get_bool/2","doc":"Return a bit out of a binary as a boolean value.","ref":"Wafer.Twiddles.html#get_bool/2"},{"type":"function","title":"Example - Wafer.Twiddles.get_bool/2","doc":"iex> get_bool(0x7f, 6)\n    true\n\n    iex> get_bool( >, 6)\n    true","ref":"Wafer.Twiddles.html#get_bool/2-example"},{"type":"macro","title":"Wafer.Twiddles.is_bit/1","doc":"","ref":"Wafer.Twiddles.html#is_bit/1"},{"type":"macro","title":"Wafer.Twiddles.is_bit_number/1","doc":"","ref":"Wafer.Twiddles.html#is_bit_number/1"},{"type":"macro","title":"Wafer.Twiddles.is_byte/1","doc":"","ref":"Wafer.Twiddles.html#is_byte/1"},{"type":"function","title":"Wafer.Twiddles.set_bit/2","doc":"Set a the specified `bit_number` to `1` within the `byte`.","ref":"Wafer.Twiddles.html#set_bit/2"},{"type":"function","title":"Example - Wafer.Twiddles.set_bit/2","doc":"iex> set_bit(0, 0)\n    1\n\n    iex> set_bit( >, 0)\n     >","ref":"Wafer.Twiddles.html#set_bit/2-example"},{"type":"function","title":"Wafer.Twiddles.set_bit/3","doc":"Set the bit at the specified `bit_number` within the `byte` to the `bit`\nvalue.","ref":"Wafer.Twiddles.html#set_bit/3"},{"type":"function","title":"Example - Wafer.Twiddles.set_bit/3","doc":"iex> set_bit(0, 1, 1)\n    2\n\n    iex> set_bit( >, 1, 1)\n     >\n\n    iex> set_bit(0, 1, true)\n    2\n\n    iex> set_bit( >, 1, true)\n     >","ref":"Wafer.Twiddles.html#set_bit/3-example"},{"type":"type","title":"Wafer.Twiddles.bit/0","doc":"","ref":"Wafer.Twiddles.html#t:bit/0"},{"type":"type","title":"Wafer.Twiddles.bit_number/0","doc":"","ref":"Wafer.Twiddles.html#t:bit_number/0"},{"type":"type","title":"Wafer.Twiddles.single_byte_binary/0","doc":"","ref":"Wafer.Twiddles.html#t:single_byte_binary/0"},{"type":"extras","title":"Wafer","doc":"# Wafer\n\n[![Build Status](https://drone.harton.nz/api/badges/james/wafer/status.svg?ref=refs/heads/main)](https://drone.harton.nz/james/wafer)\n[![Hex.pm](https://img.shields.io/hexpm/v/wafer.svg)](https://hex.pm/packages/wafer)\n[![Hippocratic License HL3-FULL](https://img.shields.io/static/v1?label=Hippocratic%20License&message=HL3-FULL&labelColor=5e2751&color=bc8c3d)](https://firstdonoharm.dev/version/3/0/full.html)\n\nWafer is an OTP application that assists with writing drivers for peripherals using I2C, SPI and GPIO pins.\n\nWafer provides Elixir protocols for interacting with device registers and dealing with GPIO, so that you can use directly connected hardware GPIO pins or GPIO expanders such as the [MCP23008](https://www.microchip.com/wwwproducts/en/MCP23008) or the [CD74HC595](http://www.ti.com/product/CD74HC595) SPI shift register.\n\nWafer implements the [GPIO](https://hexdocs.pm/wafer/Wafer.GPIOProto.html) and [Chip](https://hexdocs.pm/wafer/Wafer.Chip.html) protocols for [ElixirALE](https://hex.pm/packages/elixir_ale)'s GPIO and I2C drivers, [Circuits.GPIO](https://hex.pm/packages/circuits_gpio) and [Circuits.I2C](https://hex.pm/packages/circuits_i2c). Implementing it for SPI should also be trivial, I just don't have any SPI devices to test with at the moment.\n\nDocumentation for the main branch can always be found [here](https://docs.harton.nz/james/wafer/).\n\nSome examples of how to use this project:\n\n- [Augie](https://gitlab.com/jimsy/augie), a hexapod robot.\n- [PCA9641](https://gitlab.com/jimsy/pca9641), an example of how easy it is to write a driver with Wafer.","ref":"readme.html"},{"type":"extras","title":"Working with registers - Wafer","doc":"Wafer provides the very helpful [Registers](https://hexdocs.pm/wafer/Wafer.Registers.html) macros which allow you to quickly and easily define your registers for your device:\n\nHere's a very simple example:\n\n```elixir\ndefmodule HTS221.Registers do\n  use Wafer.Registers\n\n  defregister(:ctrl_reg1, 0x20, :rw, 1)\n  defregister(:humidity_out_l, 0x28, :ro, 1)\n  defregister(:humidity_out_h, 0x29, :ro, 1)\nend\n\ndefmodule HTS221 do\n  import HTS221.Registers\n  use Bitwise\n\n  def humidity(conn) do\n    with {:ok,  >}  }  >} -> true\n      _ -> false\n    end\n  end\n\n  def turn_on(conn), do: write_ctrl_reg1(conn,  >)\n  def turn_off(conn), do: write_ctrl_reg1(conn,  >)\nend\n```","ref":"readme.html#working-with-registers"},{"type":"extras","title":"Working with GPIO - Wafer","doc":"Wafer provides a simple way to drive specific GPIO functionality per device.\n\nHere's a super simple \"blinky\" example:\n\n```elixir\ndefmodule WaferBlinky do\n  @derive [Wafer.GPIO]\n  defstruct ~w[conn]a\n  @behaviour Wafer.Conn\n  alias Wafer.Conn\n  alias Wafer.GPIO\n\n  @type t :: %WaferBlinky{conn: Conn.t()}\n  @type acquire_options :: [acquire_option]\n  @type acquire_option :: {:conn, Conn.t()}\n\n  @impl Wafer.Conn\n  def acquire(options) do\n    with {:ok, conn}   {:error, \"`WaferBlinky.acquire/1` requires the `conn` option.\"}\n      {:error, reason} -> {:error, reason}\n    end\n  end\n\n  def turn_on(conn), do: GPIO.write(conn, 1)\n  def turn_off(conn), do: GPIO.write(conn, 0)\nend\n```\n\nAnd a simple mix task to drive it:\n\n```elixir\ndefmodule Mix.Tasks.Blink do\n  use Mix.Task\n  @shortdoc \"GPIO LED Blink Example\"\n\n  alias Wafer.Driver.Circuits.GPIO\n\n  def run(_args) do\n    {:ok, led_pin_21} = GPIO.acquire(pin: 21, direction: :out)\n    {:ok, conn} = WaferBlinky.acquire(conn: led_pin_21)\n\n    Enum.each(1..10, fn _ ->\n      WaferBlinky.turn_on(conn)\n      :timer.sleep(500)\n      WaferBlinky.turn_off(conn)\n      :timer.sleep(500)\n    end)\n  end\nend\n```","ref":"readme.html#working-with-gpio"},{"type":"extras","title":"Running the tests - Wafer","doc":"I've included stub implementations of the parts of `ElixirALE` and `Circuits`\nthat are interacted with by this project, so the tests should run and pass on\nmachines without physical hardware interfaces. If you have a Raspberry Pi with\na Pi Sense Hat connected you can run the tests with the `SENSE_HAT_PRESENT=true`\nenvironment variable set and it will perform integration tests with two of the\nsensors on this device.","ref":"readme.html#running-the-tests"},{"type":"extras","title":"Installation - Wafer","doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `wafer` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:wafer, \"~> 1.0.3\"}\n  ]\nend\n```\n\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at [https://hexdocs.pm/wafer](https://hexdocs.pm/wafer).","ref":"readme.html#installation"},{"type":"extras","title":"Gitlab Mirror - Wafer","doc":"This repository is mirrored [on Gitlab](https://gitlab.com/jimsy/wafer) from it's primary location [on my Forgejo instance](https://code.harton.nz/james/smokestack). Feel free to raise issues and open PRs on Gitlab.","ref":"readme.html#gitlab-mirror"},{"type":"extras","title":"License - Wafer","doc":"This software is licensed under the terms of the\n[HL3-FULL](https://firstdonoharm.dev), see the `LICENSE.md` file included with\nthis package for the terms.\n\nThis license actively proscribes this software being used by and for some\nindustries, countries and activities. If your usage of this software doesn't\ncomply with the terms of this license, then [contact me](mailto:james@harton.nz)\nwith the details of your use-case to organise the purchase of a license - the\ncost of which may include a donation to a suitable charity or NGO.","ref":"readme.html#license"}],"content_type":"text/markdown"}